@@ -2,7 +2,7 @@
 # Copyright (c) 2006-2016 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2010 Daniel Harding <dharding@gmail.com>
 # Copyright (c) 2012-2014 Google, Inc.
-# Copyright (c) 2013-2017 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2013-2018 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Brett Cannon <brett@python.org>
 # Copyright (c) 2014 Arun Persaud <arun@nubati.net>
 # Copyright (c) 2015 Nick Bastin <nick.bastin@gmail.com>
@@ -13,29 +13,36 @@
 # Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
 # Copyright (c) 2015 Radu Ciorba <radu@devrandom.ro>
 # Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
+# Copyright (c) 2016, 2018 Jakub Wilk <jwilk@jwilk.net>
 # Copyright (c) 2016-2017 Łukasz Rogalski <rogalski.91@gmail.com>
 # Copyright (c) 2016 Glenn Matthews <glenn@e-dad.net>
 # Copyright (c) 2016 Elias Dorneles <eliasdorneles@gmail.com>
 # Copyright (c) 2016 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2016 Yannack <yannack@users.noreply.github.com>
-# Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
 # Copyright (c) 2016 Alex Jurkiewicz <alex@jurkiewi.cz>
+# Copyright (c) 2017 Jacques Kvam <jwkvam@gmail.com>
 # Copyright (c) 2017 ttenhoeve-aa <ttenhoeve@appannie.com>
 # Copyright (c) 2017 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
+# Copyright (c) 2018 Steven M. Vascellaro <svascellaro@gmail.com>
+# Copyright (c) 2018 Mike Frysinger <vapier@gmail.com>
+# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
+# Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
+# Copyright (c) 2018 Chris Lamb <chris@chris-lamb.co.uk>
+# Copyright (c) 2018 glmdgrielson <32415403+glmdgrielson@users.noreply.github.com>
+# Copyright (c) 2018 Ville Skyttä <ville.skytta@upcloud.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
 """basic checker for Python code"""
 
+import builtins
 import collections
 import itertools
 import sys
 import re
 
-import six
-from six.moves import zip  # pylint: disable=redefined-builtin
-
 import astroid
 import astroid.bases
 import astroid.scoped_nodes
@@ -50,7 +57,7 @@ from pylint.reporters.ureports import nodes as reporter_nodes
 import pylint.utils as lint_utils
 
 
-class NamingStyle(object):
+class NamingStyle:
     # It may seem counterintuitive that single naming style
     # has multiple "accepted" forms of regular expressions,
     # but we need to special-case stuff like dunder names
@@ -79,6 +86,7 @@ class NamingStyle(object):
 
 
 class SnakeCaseStyle(NamingStyle):
+    """Regex rules for snake_case naming style."""
     CLASS_NAME_RGX = re.compile('[a-z_][a-z0-9_]+$')
     MOD_NAME_RGX = re.compile('([a-z_][a-z0-9_]*)$')
     CONST_NAME_RGX = re.compile('(([a-z_][a-z0-9_]*)|(__.*__))$')
@@ -88,6 +96,7 @@ class SnakeCaseStyle(NamingStyle):
 
 
 class CamelCaseStyle(NamingStyle):
+    """Regex rules for camelCase naming style."""
     CLASS_NAME_RGX = re.compile('[a-z_][a-zA-Z0-9]+$')
     MOD_NAME_RGX = re.compile('([a-z_][a-zA-Z0-9]*)$')
     CONST_NAME_RGX = re.compile('(([a-z_][A-Za-z0-9]*)|(__.*__))$')
@@ -97,6 +106,7 @@ class CamelCaseStyle(NamingStyle):
 
 
 class PascalCaseStyle(NamingStyle):
+    """Regex rules for PascalCase naming style."""
     CLASS_NAME_RGX = re.compile('[A-Z_][a-zA-Z0-9]+$')
     MOD_NAME_RGX = re.compile('[A-Z_][a-zA-Z0-9]+$')
     CONST_NAME_RGX = re.compile('(([A-Z_][A-Za-z0-9]*)|(__.*__))$')
@@ -106,6 +116,7 @@ class PascalCaseStyle(NamingStyle):
 
 
 class UpperCaseStyle(NamingStyle):
+    """Regex rules for UPPER_CASE naming style."""
     CLASS_NAME_RGX = re.compile('[A-Z_][A-Z0-9_]+$')
     MOD_NAME_RGX = re.compile('[A-Z_][A-Z0-9_]+$')
     CONST_NAME_RGX = re.compile('(([A-Z_][A-Z0-9_]*)|(__.*__))$')
@@ -134,11 +145,15 @@ TYPECHECK_COMPARISON_OPERATORS = frozenset(('is', 'is not', '==',
                                             '!=', 'in', 'not in'))
 LITERAL_NODE_TYPES = (astroid.Const, astroid.Dict, astroid.List, astroid.Set)
 UNITTEST_CASE = 'unittest.case'
-BUILTINS = six.moves.builtins.__name__
+BUILTINS = builtins.__name__
 TYPE_QNAME = "%s.type" % BUILTINS
 PY33 = sys.version_info >= (3, 3)
 PY3K = sys.version_info >= (3, 0)
 PY35 = sys.version_info >= (3, 5)
+ABC_METACLASSES = {
+    '_py_abc.ABCMeta', # Python 3.7+,
+    'abc.ABCMeta',
+}
 
 # Name categories that are always consistent with all naming conventions.
 EXEMPT_NAME_CATEGORIES = {'exempt', 'ignore'}
@@ -150,6 +165,7 @@ DEFAULT_ARGUMENT_SYMBOLS = dict(
         ['set()', '{}', '[]'])
 )
 REVERSED_COMPS = {'<': '>', '<=': '>=', '>': '<', '>=': '<='}
+COMPARISON_OPERATORS = frozenset(('==', '!=', '<', '>', '<=', '>='))
 
 
 def _redefines_import(node):
@@ -247,10 +263,7 @@ def _is_multi_naming_match(match, node_type, confidence):
             and (node_type != 'method' or confidence != interfaces.INFERENCE_FAILURE))
 
 
-if sys.version_info < (3, 0):
-    BUILTIN_PROPERTY = '__builtin__.property'
-else:
-    BUILTIN_PROPERTY = 'builtins.property'
+BUILTIN_PROPERTY = 'builtins.property'
 
 
 def _get_properties(config):
@@ -259,7 +272,7 @@ def _get_properties(config):
     Property classes are fully qualified, such as 'abc.abstractproperty' and
     property names are the actual names, such as 'abstract_property'.
     """
-    property_classes = set((BUILTIN_PROPERTY,))
+    property_classes = {BUILTIN_PROPERTY}
     property_names = set()  # Not returning 'property', it has its own check.
     if config is not None:
         property_classes.update(config.property_classes)
@@ -408,7 +421,7 @@ class BasicErrorChecker(_BasicChecker):
                   {'maxversion': (3, 3)}),
         'E0107': ("Use of the non-existent %s operator",
                   'nonexistent-operator',
-                  "Used when you attempt to use the C-style pre-increment or"
+                  "Used when you attempt to use the C-style pre-increment or "
                   "pre-decrement operator -- and ++, which doesn't exist in Python."),
         'E0108': ('Duplicate argument name %s in function definition',
                   'duplicate-argument-name',
@@ -426,22 +439,18 @@ class BasicErrorChecker(_BasicChecker):
         'E0112': ('More than one starred expression in assignment',
                   'too-many-star-expressions',
                   'Emitted when there are more than one starred '
-                  'expressions (`*x`) in an assignment. This is a SyntaxError.',
-                  {'minversion': (3, 0)}),
+                  'expressions (`*x`) in an assignment. This is a SyntaxError.'),
         'E0113': ('Starred assignment target must be in a list or tuple',
                   'invalid-star-assignment-target',
                   'Emitted when a star expression is used as a starred '
-                  'assignment target.',
-                  {'minversion': (3, 0)}),
+                  'assignment target.'),
         'E0114': ('Can use starred expression only in assignment target',
                   'star-needs-assignment-target',
                   'Emitted when a star expression is not used in an '
-                  'assignment target.',
-                  {'minversion': (3, 0)}),
+                  'assignment target.'),
         'E0115': ('Name %r is nonlocal and global',
                   'nonlocal-and-global',
-                  'Emitted when a name is both nonlocal and global.',
-                  {'minversion': (3, 0)}),
+                  'Emitted when a name is both nonlocal and global.'),
         'E0116': ("'continue' not supported inside 'finally' clause",
                   'continue-in-finally',
                   'Emitted when the `continue` keyword is found '
@@ -449,8 +458,7 @@ class BasicErrorChecker(_BasicChecker):
         'E0117': ("nonlocal name %s found without binding",
                   'nonlocal-without-binding',
                   'Emitted when a nonlocal variable does not have an attached '
-                  'name somewhere in the parent scopes',
-                  {'minversion': (3, 0)}),
+                  'name somewhere in the parent scopes'),
         'E0118': ("Name %r is used prior to global declaration",
                   'used-prior-global-declaration',
                   'Emitted when a name is used prior a global declaration, '
@@ -542,6 +550,9 @@ class BasicErrorChecker(_BasicChecker):
             if child.scope() is node
         }
 
+        if not scope_globals:
+            return
+
         for node_name in node.nodes_of_class(astroid.Name):
             if node_name.scope() is not node:
                 continue
@@ -565,6 +576,10 @@ class BasicErrorChecker(_BasicChecker):
         nonlocals = set(from_iter(
             child.names for child in node.nodes_of_class(astroid.Nonlocal)
             if same_scope(child)))
+
+        if not nonlocals:
+            return
+
         global_vars = set(from_iter(
             child.names for child in node.nodes_of_class(astroid.Global)
             if same_scope(child)))
@@ -659,19 +674,26 @@ class BasicErrorChecker(_BasicChecker):
             return
 
         # __init__ was called
-        metaclass = infered.metaclass()
         abstract_methods = _has_abstract_methods(infered)
+
+        if not abstract_methods:
+            return
+
+        metaclass = infered.metaclass()
+
         if metaclass is None:
             # Python 3.4 has `abc.ABC`, which won't be detected
             # by ClassNode.metaclass()
             for ancestor in infered.ancestors():
-                if ancestor.qname() == 'abc.ABC' and abstract_methods:
+                if ancestor.qname() == 'abc.ABC':
                     self.add_message('abstract-class-instantiated',
                                      args=(infered.name, ),
                                      node=node)
                     break
+
             return
-        if metaclass.qname() == 'abc.ABCMeta' and abstract_methods:
+
+        if metaclass.qname() in ABC_METACLASSES:
             self.add_message('abstract-class-instantiated',
                              args=(infered.name, ),
                              node=node)
@@ -714,7 +736,7 @@ class BasicErrorChecker(_BasicChecker):
         if defined_self is not node and not astroid.are_exclusive(node, defined_self):
             dummy_variables_rgx = lint_utils.get_global_option(
                 self, 'dummy-variables-rgx', default=None)
-            if dummy_variables_rgx and dummy_variables_rgx.match(defined_self.name):
+            if dummy_variables_rgx and dummy_variables_rgx.match(node.name):
                 return
             self.add_message('function-redefined', node=node,
                              args=(redeftype, defined_self.fromlineno))
@@ -724,7 +746,7 @@ class BasicChecker(_BasicChecker):
     """checks for :
     * doc strings
     * number of arguments, local variables, branches, returns and statements in
-functions, methods
+    functions, methods
     * required module attributes
     * dangerous default values as arguments
     * redefinition of function / method / class
@@ -806,7 +828,6 @@ functions, methods
                   'Used when the first argument to reversed() builtin '
                   'isn\'t a sequence (does not implement __reversed__, '
                   'nor __getitem__ and __len__'),
-
     }
 
     reports = (('RP0101', 'Statistics by type', report_by_type_stats),)
@@ -879,8 +900,7 @@ functions, methods
     def visit_expr(self, node):
         """check for various kind of statements without effect"""
         expr = node.value
-        if isinstance(expr, astroid.Const) and isinstance(expr.value,
-                                                          six.string_types):
+        if isinstance(expr, astroid.Const) and isinstance(expr.value, str):
             # treat string statement in a separated message
             # Handle PEP-257 attribute docstrings.
             # An attribute docstring is defined as being a string right after
@@ -892,7 +912,7 @@ functions, methods
                 else:
                     sibling = expr.previous_sibling()
                     if (sibling is not None and sibling.scope() is scope and
-                            isinstance(sibling, astroid.Assign)):
+                            isinstance(sibling, (astroid.Assign, astroid.AnnAssign))):
                         return
             self.add_message('pointless-string-statement', node=node)
             return
@@ -1133,7 +1153,7 @@ functions, methods
         try...finally statement.
         If we found before a try...finally bloc a parent which its type is
         in breaker_classes, we skip the whole check."""
-        # if self._tryfinallys is empty, we're not a in try...finally bloc
+        # if self._tryfinallys is empty, we're not an in try...finally block
         if not self._tryfinallys:
             return
         # the node could be a grand-grand...-children of the try...finally
@@ -1153,7 +1173,7 @@ functions, methods
         except utils.NoSuchArgumentError:
             pass
         else:
-            if argument is astroid.YES:
+            if argument is astroid.Uninferable:
                 return
             if argument is None:
                 # Nothing was infered.
@@ -1173,8 +1193,8 @@ functions, methods
                         utils.is_builtin_object(argument._proxied)):
                     self.add_message('bad-reversed-sequence', node=node)
                     return
-                elif any(ancestor.name == 'dict' and utils.is_builtin_object(ancestor)
-                         for ancestor in argument._proxied.ancestors()):
+                if any(ancestor.name == 'dict' and utils.is_builtin_object(ancestor)
+                       for ancestor in argument._proxied.ancestors()):
                     # Mappings aren't accepted by reversed(), unless
                     # they provide explicitly a __reversed__ method.
                     try:
@@ -1260,7 +1280,7 @@ DEFAULT_NAMING_STYLES = {
 
 def _create_naming_options():
     name_options = []
-    for name_type in KNOWN_NAME_TYPES:
+    for name_type in sorted(KNOWN_NAME_TYPES):
         human_readable_name = HUMAN_READABLE_TYPES[name_type]
         default_style = DEFAULT_NAMING_STYLES[name_type]
         name_type = name_type.replace('_', '-')
@@ -1268,11 +1288,11 @@ def _create_naming_options():
             '%s-naming-style' % (name_type,),
             {'default': default_style,
              'type': 'choice', 'choices': list(NAMING_STYLES.keys()), 'metavar': '<style>',
-             'help': 'Naming style matching correct %s names' % (human_readable_name,)}),)
+             'help': 'Naming style matching correct %s names.' % (human_readable_name,)}),)
         name_options.append((
             '%s-rgx' % (name_type,),
             {'default': None, 'type': 'regexp', 'metavar': '<regexp>',
-             'help': 'Regular expression matching correct %s names. Overrides %s-naming-style'
+             'help': 'Regular expression matching correct %s names. Overrides %s-naming-style.'
                      % (human_readable_name, name_type,)}))
     return tuple(name_options)
 
@@ -1291,20 +1311,20 @@ class NameChecker(_BasicChecker):
         'W0111': ('Name %s will become a keyword in Python %s',
                   'assign-to-new-keyword',
                   'Used when assignment will become invalid in future '
-                  'Python release due to introducing new keyword'),
+                  'Python release due to introducing new keyword.'),
     }
 
     options = (('good-names',
                 {'default' : ('i', 'j', 'k', 'ex', 'Run', '_'),
                  'type' :'csv', 'metavar' : '<names>',
                  'help' : 'Good variable names which should always be accepted,'
-                          ' separated by a comma'}
+                          ' separated by a comma.'}
                ),
                ('bad-names',
                 {'default' : ('foo', 'bar', 'baz', 'toto', 'tutu', 'tata'),
                  'type' :'csv', 'metavar' : '<names>',
                  'help' : 'Bad variable names which should always be refused, '
-                          'separated by a comma'}
+                          'separated by a comma.'}
                ),
                ('name-group',
                 {'default' : (),
@@ -1315,7 +1335,7 @@ class NameChecker(_BasicChecker):
                ),
                ('include-naming-hint',
                 {'default': False, 'type': 'yn', 'metavar': '<y_or_n>',
-                 'help': 'Include a hint for the correct naming format with invalid-name'}
+                 'help': 'Include a hint for the correct naming format with invalid-name.'}
                ),
                ('property-classes',
                 {'default': ('abc.abstractproperty',),
@@ -1328,7 +1348,6 @@ class NameChecker(_BasicChecker):
               ) + _create_naming_options()
 
     KEYWORD_ONSET = {
-        (3, 0): {'True', 'False'},
         (3, 7): {'async', 'await'}
     }
 
@@ -1385,12 +1404,12 @@ class NameChecker(_BasicChecker):
         self._bad_names = {}
 
     def leave_module(self, node): # pylint: disable=unused-argument
-        for all_groups in six.itervalues(self._bad_names):
+        for all_groups in self._bad_names.values():
             if len(all_groups) < 2:
                 continue
             groups = collections.defaultdict(list)
             min_warnings = sys.maxsize
-            for group in six.itervalues(all_groups):
+            for group in all_groups.values():
                 groups[len(group)].append(group)
                 min_warnings = min(len(group), min_warnings)
             if len(groups[min_warnings]) > 1:
@@ -1406,7 +1425,7 @@ class NameChecker(_BasicChecker):
     def visit_classdef(self, node):
         self._check_assign_to_new_keyword_violation(node.name, node)
         self._check_name('class', node.name, node)
-        for attr, anodes in six.iteritems(node.instance_attrs):
+        for attr, anodes in node.instance_attrs.items():
             if not any(node.instance_attr_ancestors(attr)):
                 self._check_name('attr', attr, anodes[0])
 
@@ -1442,19 +1461,19 @@ class NameChecker(_BasicChecker):
         """check module level assigned names"""
         self._check_assign_to_new_keyword_violation(node.name, node)
         frame = node.frame()
-        ass_type = node.assign_type()
-        if isinstance(ass_type, astroid.Comprehension):
+        assign_type = node.assign_type()
+        if isinstance(assign_type, astroid.Comprehension):
             self._check_name('inlinevar', node.name, node)
         elif isinstance(frame, astroid.Module):
-            if isinstance(ass_type, astroid.Assign) and not in_loop(ass_type):
-                if isinstance(utils.safe_infer(ass_type.value), astroid.ClassDef):
+            if isinstance(assign_type, astroid.Assign) and not in_loop(assign_type):
+                if isinstance(utils.safe_infer(assign_type.value), astroid.ClassDef):
                     self._check_name('class', node.name, node)
                 else:
                     if not _redefines_import(node):
                         # Don't emit if the name redefines an import
                         # in an ImportError except handler.
                         self._check_name('const', node.name, node)
-            elif isinstance(ass_type, astroid.ExceptHandler):
+            elif isinstance(assign_type, astroid.ExceptHandler):
                 self._check_name('variable', node.name, node)
         elif isinstance(frame, astroid.FunctionDef):
             # global introduced variable aren't in the function locals
@@ -1493,6 +1512,13 @@ class NameChecker(_BasicChecker):
 
     def _check_name(self, node_type, name, node, confidence=interfaces.HIGH):
         """check for a name using the type's regexp"""
+        def _should_exempt_from_invalid_name(node):
+            if node_type == 'variable':
+                inferred = utils.safe_infer(node)
+                if isinstance(inferred, astroid.ClassDef):
+                    return True
+            return False
+
         if utils.is_inside_except(node):
             clobbering, _ = utils.clobber_in_except(node)
             if clobbering:
@@ -1512,7 +1538,7 @@ class NameChecker(_BasicChecker):
             warnings = bad_name_group.setdefault(match.lastgroup, [])
             warnings.append((node, node_type, name, confidence))
 
-        if match is None:
+        if match is None and not _should_exempt_from_invalid_name(node):
             self._raise_name_warning(node, node_type, name, confidence)
 
     def _check_assign_to_new_keyword_violation(self, name, node):
@@ -1603,8 +1629,10 @@ class DocStringChecker(_BasicChecker):
                 self._check_docstring(ftype, node,
                                       report_missing=not overridden,
                                       confidence=confidence)
-            else:
+            elif isinstance(node.parent.frame(), astroid.Module):
                 self._check_docstring(ftype, node)
+            else:
+                return
 
     visit_asyncfunctiondef = visit_functiondef
 
@@ -1635,7 +1663,7 @@ class DocStringChecker(_BasicChecker):
                     # Strings in Python 3, others in Python 2.
                     if PY3K and func.bound.name == 'str':
                         return
-                    elif func.bound.name in ('str', 'unicode', 'bytes'):
+                    if func.bound.name in ('str', 'unicode', 'bytes'):
                         return
             self.add_message('missing-docstring', node=node, args=(node_type,),
                              confidence=confidence)
@@ -1726,23 +1754,41 @@ class ComparisonChecker(_BasicChecker):
                       'Used when comparing an object to a literal, which is usually '
                       'what you do not want to do, since you can compare to a different '
                       'literal than what was expected altogether.'),
-           }
+            'R0124': ('Redundant comparison - %s',
+                      'comparison-with-itself',
+                      'Used when something is compared against itself.'),
+            'W0143': ('Comparing against a callable, did you omit the parenthesis?',
+                      'comparison-with-callable',
+                      'This message is emitted when pylint detects that a comparison with a '
+                      'callable was made, which might suggest that some parenthesis were omitted, '
+                      'resulting in potential unwanted behaviour.'),
+            }
 
-    def _check_singleton_comparison(self, singleton, root_node):
+    def _check_singleton_comparison(self, singleton, root_node, negative_check=False):
         if singleton.value is True:
-            suggestion = "just 'expr' or 'expr is True'"
+            if not negative_check:
+                suggestion = "just 'expr' or 'expr is True'"
+            else:
+                suggestion = "just 'not expr' or 'expr is False'"
             self.add_message('singleton-comparison',
                              node=root_node,
                              args=(True, suggestion))
         elif singleton.value is False:
-            suggestion = "'not expr' or 'expr is False'"
+            if not negative_check:
+                suggestion = "'not expr' or 'expr is False'"
+            else:
+                suggestion = "'expr' or 'expr is not False'"
             self.add_message('singleton-comparison',
                              node=root_node,
                              args=(False, suggestion))
         elif singleton.value is None:
+            if not negative_check:
+                suggestion = "'expr is None'"
+            else:
+                suggestion = "'expr is not None'"
             self.add_message('singleton-comparison',
                              node=root_node,
-                             args=(None, "'expr is None'"))
+                             args=(None, suggestion))
 
     def _check_literal_comparison(self, literal, node):
         """Check if we compare to a literal, which is usually what we do not want to do."""
@@ -1769,9 +1815,50 @@ class ComparisonChecker(_BasicChecker):
         self.add_message('misplaced-comparison-constant', node=node,
                          args=(suggestion,))
 
+    def _check_logical_tautology(self, node):
+        """Check if identifier is compared against itself.
+        :param node: Compare node
+        :type node: astroid.node_classes.Compare
+        :Example:
+        val = 786
+        if val == val:  # [comparison-with-itself]
+            pass
+        """
+        left_operand = node.left
+        right_operand = node.ops[0][1]
+        operator = node.ops[0][0]
+        if (isinstance(left_operand, astroid.Const)
+                and isinstance(right_operand, astroid.Const)):
+            left_operand = left_operand.value
+            right_operand = right_operand.value
+        elif (isinstance(left_operand, astroid.Name)
+              and isinstance(right_operand, astroid.Name)):
+            left_operand = left_operand.name
+            right_operand = right_operand.name
+
+        if left_operand == right_operand:
+            suggestion = "%s %s %s" % (left_operand, operator, right_operand)
+            self.add_message('comparison-with-itself', node=node, args=(suggestion,))
+
+    def _check_callable_comparison(self, node):
+        operator = node.ops[0][0]
+        if operator not in COMPARISON_OPERATORS:
+            return
+
+        bare_callables = (astroid.FunctionDef, astroid.BoundMethod)
+        left_operand, right_operand = node.left, node.ops[0][1]
+        # this message should be emitted only when there is comparison of bare callable
+        # with non bare callable.
+        if sum([1 for operand in (left_operand, right_operand)
+                if isinstance(utils.safe_infer(operand), bare_callables)]) == 1:
+            self.add_message('comparison-with-callable', node=node)
+
     @utils.check_messages('singleton-comparison', 'misplaced-comparison-constant',
-                          'unidiomatic-typecheck', 'literal-comparison')
+                          'unidiomatic-typecheck', 'literal-comparison', 'comparison-with-itself',
+                          'comparison-with-callable')
     def visit_compare(self, node):
+        self._check_callable_comparison(node)
+        self._check_logical_tautology(node)
         self._check_unidiomatic_typecheck(node)
         # NOTE: this checker only works with binary comparisons like 'x == 42'
         # but not 'x == y == 42'
@@ -1780,7 +1867,7 @@ class ComparisonChecker(_BasicChecker):
 
         left = node.left
         operator, right = node.ops[0]
-        if (operator in ('<', '<=', '>', '>=', '!=', '==')
+        if (operator in COMPARISON_OPERATORS
                 and isinstance(left, astroid.Const)):
             self._check_misplaced_constant(node, left, right, operator)
 
@@ -1789,6 +1876,9 @@ class ComparisonChecker(_BasicChecker):
                 self._check_singleton_comparison(left, node)
             elif isinstance(right, astroid.Const):
                 self._check_singleton_comparison(right, node)
+        if operator == '!=':
+            if isinstance(right, astroid.Const):
+                self._check_singleton_comparison(right, node, negative_check=True)
         if operator in ('is', 'is not'):
             self._check_literal_comparison(right, node)
 
