@@ -16,11 +16,20 @@
 # Copyright (c) 2016 Petr Pulc <petrpulc@gmail.com>
 # Copyright (c) 2016 Moises Lopez <moylop260@vauxoo.com>
 # Copyright (c) 2016 Ashley Whetter <ashley@awhetter.co.uk>
+# Copyright (c) 2017-2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
 # Copyright (c) 2017 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2017 Krzysztof Czapla <k.czapla68@gmail.com>
 # Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
 # Copyright (c) 2017 James M. Allen <james.m.allen@gmail.com>
 # Copyright (c) 2017 vinnyrose <vinnyrose@users.noreply.github.com>
+# Copyright (c) 2018 Bryce Guinta <bryce.guinta@protonmail.com>
+# Copyright (c) 2018 Mike Frysinger <vapier@gmail.com>
+# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
+# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2018 Fureigh <rhys.fureigh@gsa.gov>
+# Copyright (c) 2018 Pierre Sassoulas <pierre.sassoulas@wisebim.fr>
+# Copyright (c) 2018 Andreas Freimuth <andreas.freimuth@united-bits.de>
+# Copyright (c) 2018 Jakub Wilk <jwilk@jwilk.net>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -29,7 +38,7 @@
 
 By default try to follow Guido's style guide :
 
-http://www.python.org/doc/essays/styleguide.html
+https://www.python.org/doc/essays/styleguide/
 
 Some parts of the process_token method is based from The Tab Nanny std module.
 """
@@ -37,10 +46,7 @@ Some parts of the process_token method is based from The Tab Nanny std module.
 import keyword
 import sys
 import tokenize
-from functools import reduce # pylint: disable=redefined-builtin
-
-import six
-from six.moves import zip, map, filter # pylint: disable=redefined-builtin
+from functools import reduce  # pylint: disable=redefined-builtin
 
 from astroid import nodes
 
@@ -50,9 +56,9 @@ from pylint.checkers.utils import check_messages
 from pylint.utils import WarningScope, OPTION_RGX
 
 _ASYNC_TOKEN = 'async'
-_CONTINUATION_BLOCK_OPENERS = ['elif', 'except', 'for', 'if', 'while', 'def', 'class']
+_CONTINUATION_BLOCK_OPENERS = ['elif', 'except', 'for', 'if', 'while', 'def', 'class', 'with']
 _KEYWORD_TOKENS = ['assert', 'del', 'elif', 'except', 'for', 'if', 'in', 'not',
-                   'raise', 'return', 'while', 'yield']
+                   'raise', 'return', 'while', 'yield', 'with']
 if sys.version_info < (3, 0):
     _KEYWORD_TOKENS.append('print')
 
@@ -82,9 +88,9 @@ MSGS = {
     'C0301': ('Line too long (%s/%s)',
               'line-too-long',
               'Used when a line is longer than a given number of characters.'),
-    'C0302': ('Too many lines in module (%s/%s)', # was W0302
+    'C0302': ('Too many lines in module (%s/%s)',  # was W0302
               'too-many-lines',
-              'Used when a module has too much lines, reducing its readability.'
+              'Used when a module has too many lines, reducing its readability.'
              ),
     'C0303': ('Trailing whitespace',
               'trailing-whitespace',
@@ -106,7 +112,7 @@ MSGS = {
     'W0312': ('Found indentation with %ss instead of %ss',
               'mixed-indentation',
               'Used when there are some mixed tabs and spaces in a module.'),
-    'W0301': ('Unnecessary semicolon', # was W0106
+    'W0301': ('Unnecessary semicolon',  # was W0106
               'unnecessary-semicolon',
               'Used when a statement is ended by a semi-colon (";"), which \
               isn\'t necessary (that\'s python, not C ;).'),
@@ -114,10 +120,10 @@ MSGS = {
               'multiple-statements',
               'Used when more than on statement are found on the same line.',
               {'scope': WarningScope.NODE}),
-    'C0325' : ('Unnecessary parens after %r keyword',
-               'superfluous-parens',
-               'Used when a single item in parentheses follows an if, for, or '
-               'other keyword.'),
+    'C0325': ('Unnecessary parens after %r keyword',
+              'superfluous-parens',
+              'Used when a single item in parentheses follows an if, for, or '
+              'other keyword.'),
     'C0326': ('%s space %s %s %s\n%s',
               'bad-whitespace',
               ('Used when a wrong number of spaces is used around an operator, '
@@ -128,7 +134,7 @@ MSGS = {
     'W0332': ('Use of "l" as long integer identifier',
               'lowercase-l-suffix',
               'Used when a lower case "l" is used to mark a long integer. You '
-              'should use a upper case "L" since the letter "l" looks too much '
+              'should use an upper case "L" since the letter "l" looks too much '
               'like the digit "1"',
               {'maxversion': (3, 0)}),
     'C0327': ('Mixed line endings LF and CRLF',
@@ -149,6 +155,7 @@ def _underline_token(token):
         referenced_line += '\n'
     return referenced_line + (' ' * offset) + ('^' * length)
 
+
 def _column_distance(token1, token2):
     if token1 == token2:
         return 0
@@ -171,6 +178,17 @@ def _token_followed_by_eol(tokens, position):
             tokens.type(position+2) == tokenize.NL)
 
 
+def _get_indent_string(line):
+    """Return the indention string of the given line."""
+    result = ''
+    for char in line:
+        if char in ' \t':
+            result += char
+        else:
+            break
+    return result
+
+
 def _get_indent_length(line):
     """Return the length of the indentation on the given token's line."""
     result = 0
@@ -188,6 +206,9 @@ def _get_indent_hint_line(bar_positions, bad_position):
     """Return a line with |s for each of the positions in the given lists."""
     if not bar_positions:
         return ('', '')
+    # TODO tabs should not be replaced by some random (8) number of spaces
+    bar_positions = [_get_indent_length(indent) for indent in bar_positions]
+    bad_position = _get_indent_length(bad_position)
     delta_message = ''
     markers = [(pos, '|') for pos in bar_positions]
     if len(markers) == 1:
@@ -205,9 +226,9 @@ def _get_indent_hint_line(bar_positions, bad_position):
     return (''.join(line), delta_message)
 
 
-class _ContinuedIndent(object):
-    __slots__ = ('valid_outdent_offsets',
-                 'valid_continuation_offsets',
+class _ContinuedIndent:
+    __slots__ = ('valid_outdent_strings',
+                 'valid_continuation_strings',
                  'context_type',
                  'token',
                  'position')
@@ -216,10 +237,10 @@ class _ContinuedIndent(object):
                  context_type,
                  token,
                  position,
-                 valid_outdent_offsets,
-                 valid_continuation_offsets):
-        self.valid_outdent_offsets = valid_outdent_offsets
-        self.valid_continuation_offsets = valid_continuation_offsets
+                 valid_outdent_strings,
+                 valid_continuation_strings):
+        self.valid_outdent_strings = valid_outdent_strings
+        self.valid_continuation_strings = valid_continuation_strings
         self.context_type = context_type
         self.position = position
         self.token = token
@@ -250,16 +271,17 @@ _CONTINUATION_MSG_PARTS = {
 
 _CONTINUATION_HINT_MESSAGE = ' (%s %d space%s)'  # Ex: (remove 2 spaces)
 
-def _Offsets(*args):
-    """Valid indentation offsets for a continued line."""
-    return dict((a, None) for a in args)
+
+def _Indentations(*args):
+    """Valid indentation strings for a continued line."""
+    return {a: None for a in args}
 
 
-def _BeforeBlockOffsets(single, with_body):
-    """Valid alternative indent offsets for continued lines before blocks.
+def _BeforeBlockIndentations(single, with_body):
+    """Valid alternative indentation strings for continued lines before blocks.
 
-    :param int single: Valid offset for statements on a single logical line.
-    :param int with_body: Valid offset for statements on several lines.
+    :param int single: Valid indentation string for statements on a single logical line.
+    :param int with_body: Valid indentation string for statements on several lines.
 
     :returns: A dictionary mapping indent offsets to a string representing
         whether the indent if for a line or block.
@@ -268,7 +290,7 @@ def _BeforeBlockOffsets(single, with_body):
     return {single: SINGLE_LINE, with_body: WITH_BODY}
 
 
-class TokenWrapper(object):
+class TokenWrapper:
     """A wrapper for readable access to token information."""
 
     def __init__(self, tokens):
@@ -289,8 +311,23 @@ class TokenWrapper(object):
     def line(self, idx):
         return self._tokens[idx][4]
 
+    def line_indent(self, idx):
+        """Get the string of TABs and Spaces used for indentation of the line of this token"""
+        return _get_indent_string(self.line(idx))
 
-class ContinuedLineState(object):
+    def token_indent(self, idx):
+        """Get an indentation string for hanging indentation, consisting of the line-indent plus
+        a number of spaces to fill up to the column of this token.
+
+        e.g. the token indent for foo
+        in "<TAB><TAB>print(foo)"
+        is "<TAB><TAB>      "
+        """
+        line_indent = self.line_indent(idx)
+        return line_indent + ' ' * (self.start_col(idx) - len(line_indent))
+
+
+class ContinuedLineState:
     """Tracker for continued indentation inside a logical line."""
 
     def __init__(self, tokens, config):
@@ -306,8 +343,12 @@ class ContinuedLineState(object):
         return bool(self._cont_stack)
 
     @property
-    def _block_indent_size(self):
-        return len(self._config.indent_string.replace('\t', ' ' * _TAB_LENGTH))
+    def _block_indent_string(self):
+        return self._config.indent_string.replace('\\t', '\t')
+
+    @property
+    def _continuation_string(self):
+        return self._block_indent_string[0] * self._config.indent_after_paren
 
     @property
     def _continuation_size(self):
@@ -340,10 +381,10 @@ class ContinuedLineState(object):
         self.retained_warnings = []
         self._cont_stack = []
 
-    def add_block_warning(self, token_position, state, valid_offsets):
-        self.retained_warnings.append((token_position, state, valid_offsets))
+    def add_block_warning(self, token_position, state, valid_indentations):
+        self.retained_warnings.append((token_position, state, valid_indentations))
 
-    def get_valid_offsets(self, idx):
+    def get_valid_indentations(self, idx):
         """Returns the valid offsets for the token at the given position."""
         # The closing brace on a dict or the 'for' in a dict comprehension may
         # reset two indent levels because the dict value is ended implicitly
@@ -352,30 +393,39 @@ class ContinuedLineState(object):
             stack_top = -2
         indent = self._cont_stack[stack_top]
         if self._tokens.token(idx) in _CLOSING_BRACKETS:
-            valid_offsets = indent.valid_outdent_offsets
+            valid_indentations = indent.valid_outdent_strings
         else:
-            valid_offsets = indent.valid_continuation_offsets
-        return indent, valid_offsets.copy()
+            valid_indentations = indent.valid_continuation_strings
+        return indent, valid_indentations.copy()
 
     def _hanging_indent_after_bracket(self, bracket, position):
-        """Extracts indentation information for a hanging indent."""
-        indentation = _get_indent_length(self._tokens.line(position))
-        if self._is_block_opener and self._continuation_size == self._block_indent_size:
+        """Extracts indentation information for a hanging indent
+
+        Case of hanging indent after a bracket (including parenthesis)
+
+        :param str bracket: bracket in question
+        :param int position: Position of bracket in self._tokens
+
+        :returns: the state and valid positions for hanging indentation
+        :rtype: _ContinuedIndent
+        """
+        indentation = self._tokens.line_indent(position)
+        if self._is_block_opener and self._continuation_string == self._block_indent_string:
             return _ContinuedIndent(
                 HANGING_BLOCK,
                 bracket,
                 position,
-                _Offsets(indentation + self._continuation_size, indentation),
-                _BeforeBlockOffsets(indentation + self._continuation_size,
-                                    indentation + self._continuation_size * 2))
+                _Indentations(indentation + self._continuation_string, indentation),
+                _BeforeBlockIndentations(indentation + self._continuation_string,
+                                         indentation + self._continuation_string * 2))
         if bracket == ':':
             # If the dict key was on the same line as the open brace, the new
             # correct indent should be relative to the key instead of the
             # current indent level
-            paren_align = self._cont_stack[-1].valid_outdent_offsets
-            next_align = self._cont_stack[-1].valid_continuation_offsets.copy()
+            paren_align = self._cont_stack[-1].valid_outdent_strings
+            next_align = self._cont_stack[-1].valid_continuation_strings.copy()
             next_align_keys = list(next_align.keys())
-            next_align[next_align_keys[0] + self._continuation_size] = True
+            next_align[next_align_keys[0] + self._continuation_string] = True
             # Note that the continuation of
             # d = {
             #       'a': 'b'
@@ -387,27 +437,29 @@ class ContinuedLineState(object):
             HANGING,
             bracket,
             position,
-            _Offsets(indentation, indentation + self._continuation_size),
-            _Offsets(indentation + self._continuation_size))
+            _Indentations(indentation, indentation + self._continuation_string),
+            _Indentations(indentation + self._continuation_string))
 
-    def _continuation_inside_bracket(self, bracket, pos):
+    def _continuation_inside_bracket(self, bracket, position):
         """Extracts indentation information for a continued indent."""
-        indentation = _get_indent_length(self._tokens.line(pos))
-        token_start = self._tokens.start_col(pos)
-        next_token_start = self._tokens.start_col(pos + 1)
-        if self._is_block_opener and next_token_start - indentation == self._block_indent_size:
+        indentation = self._tokens.line_indent(position)
+        token_indent = self._tokens.token_indent(position)
+        next_token_indent = self._tokens.token_indent(position + 1)
+        if self._is_block_opener and next_token_indent == indentation + self._block_indent_string:
             return _ContinuedIndent(
                 CONTINUED_BLOCK,
                 bracket,
-                pos,
-                _Offsets(token_start),
-                _BeforeBlockOffsets(next_token_start, next_token_start + self._continuation_size))
+                position,
+                _Indentations(token_indent),
+                _BeforeBlockIndentations(
+                    next_token_indent,
+                    next_token_indent + self._continuation_string))
         return _ContinuedIndent(
             CONTINUED,
             bracket,
-            pos,
-            _Offsets(token_start),
-            _Offsets(next_token_start))
+            position,
+            _Indentations(token_indent, next_token_indent),
+            _Indentations(next_token_indent))
 
     def pop_token(self):
         self._cont_stack.pop()
@@ -450,22 +502,22 @@ class FormatChecker(BaseTokenChecker):
     # configuration options
     # for available dict keys/values see the optik parser 'add_option' method
     options = (('max-line-length',
-                {'default' : 100, 'type' : "int", 'metavar' : '<int>',
-                 'help' : 'Maximum number of characters on a single line.'}),
+                {'default': 100, 'type': "int", 'metavar': '<int>',
+                 'help': 'Maximum number of characters on a single line.'}),
                ('ignore-long-lines',
                 {'type': 'regexp', 'metavar': '<regexp>',
                  'default': r'^\s*(# )?<?https?://\S+>?$',
                  'help': ('Regexp for a line that is allowed to be longer than '
                           'the limit.')}),
                ('single-line-if-stmt',
-                {'default': False, 'type' : 'yn', 'metavar' : '<y_or_n>',
-                 'help' : ('Allow the body of an if to be on the same '
-                           'line as the test if there is no else.')}),
+                {'default': False, 'type': 'yn', 'metavar': '<y_or_n>',
+                 'help': ('Allow the body of an if to be on the same '
+                          'line as the test if there is no else.')}),
                ('single-line-class-stmt',
-                {'default': False, 'type' : 'yn', 'metavar' : '<y_or_n>',
-                 'help' : ('Allow the body of a class to be on the same '
-                           'line as the declaration if body contains '
-                           'single statement.')}),
+                {'default': False, 'type': 'yn', 'metavar': '<y_or_n>',
+                 'help': ('Allow the body of a class to be on the same '
+                          'line as the declaration if body contains '
+                          'single statement.')}),
                ('no-space-check',
                 {'default': ','.join(_DEFAULT_NO_SPACE_CHECK_CHOICES),
                  'metavar': ','.join(_NO_SPACE_CHECK_CHOICES),
@@ -479,13 +531,13 @@ class FormatChecker(BaseTokenChecker):
                           'and closing bracket: (a, ). '
                           '`'+ _EMPTY_LINE + '` allows space-only lines.')}),
                ('max-module-lines',
-                {'default' : 1000, 'type' : 'int', 'metavar' : '<int>',
-                 'help': 'Maximum number of lines in a module'}
+                {'default': 1000, 'type': 'int', 'metavar': '<int>',
+                 'help': 'Maximum number of lines in a module.'}
                ),
                ('indent-string',
-                {'default' : '    ', 'type' : "non_empty_string", 'metavar' : '<string>',
-                 'help' : 'String used as indentation unit. This is usually '
-                          '"    " (4 spaces) or "\\t" (1 tab).'}),
+                {'default': '    ', 'type': "non_empty_string", 'metavar': '<string>',
+                 'help': 'String used as indentation unit. This is usually '
+                         '"    " (4 spaces) or "\\t" (1 tab).'}),
                ('indent-after-paren',
                 {'type': 'int', 'metavar': '<int>', 'default': 4,
                  'help': 'Number of spaces of indent required inside a hanging '
@@ -588,7 +640,7 @@ class FormatChecker(BaseTokenChecker):
                     return
                 # 'and' and 'or' are the only boolean operators with lower precedence
                 # than 'not', so parens are only required when they are found.
-                elif token[1] in ('and', 'or'):
+                if token[1] in ('and', 'or'):
                     found_and_or = True
                 # A yield inside an expression must always be in parentheses,
                 # quit early without error.
@@ -632,6 +684,9 @@ class FormatChecker(BaseTokenChecker):
         """Extended check of PEP-484 type hint presence"""
         if not self._inside_brackets('('):
             return False
+        # token_info
+        # type string start end line
+        #  0      1     2    3    4
         bracket_level = 0
         for token in tokens[i-1::-1]:
             if token[1] == ':':
@@ -645,9 +700,9 @@ class FormatChecker(BaseTokenChecker):
             elif token[1] == ',':
                 if not bracket_level:
                     return False
-            elif token[1] == '.':
+            elif token[1] in ('.', '...'):
                 continue
-            elif token[0] not in (tokenize.NAME, tokenize.STRING):
+            elif token[0] not in (tokenize.NAME, tokenize.STRING, tokenize.NL):
                 return False
         return False
 
@@ -739,7 +794,7 @@ class FormatChecker(BaseTokenChecker):
             count, state = _policy_string(policy)
             self.add_message('bad-whitespace', line=token[2][0],
                              args=(count, state, position, construct,
-                                   _underline_token(token)))
+                                   _underline_token(token)), col_offset=token[2][1])
 
     def _inside_brackets(self, left):
         return self._bracket_stack[-1] == left
@@ -855,7 +910,7 @@ class FormatChecker(BaseTokenChecker):
         if line_num > self.config.max_module_lines:
             # Get the line where the too-many-lines (or its message id)
             # was disabled or default to 1.
-            symbol = self.linter.msgs_store.check_message_id('too-many-lines')
+            symbol = self.linter.msgs_store.get_message_definition('too-many-lines')
             names = (symbol.msgid, 'too-many-lines')
             line = next(filter(None,
                                map(self.linter._pragma_lineno.get, names)), 1)
@@ -889,10 +944,9 @@ class FormatChecker(BaseTokenChecker):
     def _process_retained_warnings(self, tokens, current_pos):
         single_line_block_stmt = not _last_token_on_line_is(tokens, current_pos, ':')
 
-        for indent_pos, state, offsets in self._current_line.retained_warnings:
-            block_type = offsets[tokens.start_col(indent_pos)]
-            hints = dict((k, v) for k, v in six.iteritems(offsets)
-                         if v != block_type)
+        for indent_pos, state, indentations in self._current_line.retained_warnings:
+            block_type = indentations[tokens.token_indent(indent_pos)]
+            hints = {k: v for k, v in indentations.items() if v != block_type}
             if single_line_block_stmt and block_type == WITH_BODY:
                 self._add_continuation_message(state, hints, tokens, indent_pos)
             elif not single_line_block_stmt and block_type == SINGLE_LINE:
@@ -907,27 +961,27 @@ class FormatChecker(BaseTokenChecker):
         if not self._current_line.has_content or tokens.type(next_idx) == tokenize.NL:
             return
 
-        state, valid_offsets = self._current_line.get_valid_offsets(next_idx)
+        state, valid_indentations = self._current_line.get_valid_indentations(next_idx)
         # Special handling for hanging comments and strings. If the last line ended
         # with a comment (string) and the new line contains only a comment, the line
         # may also be indented to the start of the previous token.
         if same_token_around_nl(tokenize.COMMENT) or same_token_around_nl(tokenize.STRING):
-            valid_offsets[tokens.start_col(next_idx-2)] = True
+            valid_indentations[tokens.token_indent(next_idx-2)] = True
 
         # We can only decide if the indentation of a continued line before opening
         # a new block is valid once we know of the body of the block is on the
         # same line as the block opener. Since the token processing is single-pass,
         # emitting those warnings is delayed until the block opener is processed.
         if (state.context_type in (HANGING_BLOCK, CONTINUED_BLOCK)
-                and tokens.start_col(next_idx) in valid_offsets):
-            self._current_line.add_block_warning(next_idx, state, valid_offsets)
-        elif tokens.start_col(next_idx) not in valid_offsets:
+                and tokens.token_indent(next_idx) in valid_indentations):
+            self._current_line.add_block_warning(next_idx, state, valid_indentations)
+        elif tokens.token_indent(next_idx) not in valid_indentations:
+            self._add_continuation_message(state, valid_indentations, tokens, next_idx)
 
-            self._add_continuation_message(state, valid_offsets, tokens, next_idx)
-
-    def _add_continuation_message(self, state, offsets, tokens, position):
+    def _add_continuation_message(self, state, indentations, tokens, position):
         readable_type, readable_position = _CONTINUATION_MSG_PARTS[state.context_type]
-        hint_line, delta_message = _get_indent_hint_line(offsets, tokens.start_col(position))
+        hint_line, delta_message = _get_indent_hint_line(indentations,
+                                                         tokens.token_indent(position))
         self.add_message(
             'bad-continuation',
             line=tokens.start_line(position),
@@ -1010,14 +1064,16 @@ class FormatChecker(BaseTokenChecker):
                     # allow empty lines
                     pass
                 elif line[len(stripped_line):] not in ('\n', '\r\n'):
-                    self.add_message('trailing-whitespace', line=i)
+                    self.add_message('trailing-whitespace', line=i,
+                                     col_offset=len(stripped_line))
                 # Don't count excess whitespace in the line length.
                 line = stripped_line
             mobj = OPTION_RGX.search(line)
             if mobj and '=' in line:
-                front_of_equal, back_of_equal = mobj.group(1).split('=', 1)
+                front_of_equal, _, back_of_equal = mobj.group(1).partition('=')
                 if front_of_equal.strip() == 'disable':
-                    if 'line-too-long' in [_msg_id.strip() for _msg_id in back_of_equal.split(',')]:
+                    if 'line-too-long' in {_msg_id.strip()
+                                           for _msg_id in back_of_equal.split(',')}:
                         return None
                     line = line.rsplit('#', 1)[0].rstrip()
 
@@ -1026,16 +1082,16 @@ class FormatChecker(BaseTokenChecker):
             return i + 1
 
         unsplit_ends = {
-            u'\v',
-            u'\x0b',
-            u'\f',
-            u'\x0c',
-            u'\x1c',
-            u'\x1d',
-            u'\x1e',
-            u'\x85',
-            u'\u2028',
-            u'\u2029'
+            '\v',
+            '\x0b',
+            '\f',
+            '\x0c',
+            '\x1c',
+            '\x1d',
+            '\x1e',
+            '\x85',
+            '\u2028',
+            '\u2029'
         }
         unsplit = []
         for line in lines.splitlines(True):
