@@ -4,7 +4,7 @@
 # Copyright (c) 2010 Daniel Harding <dharding@gmail.com>
 # Copyright (c) 2012-2014 Google, Inc.
 # Copyright (c) 2012 FELD Boris <lothiraldan@gmail.com>
-# Copyright (c) 2013-2017 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2013-2018 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Brett Cannon <brett@python.org>
 # Copyright (c) 2014 Ricardo Gemignani <ricardo.gemignani@gmail.com>
 # Copyright (c) 2014 Arun Persaud <arun@nubati.net>
@@ -12,12 +12,16 @@
 # Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
 # Copyright (c) 2015 Radu Ciorba <radu@devrandom.ro>
 # Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
+# Copyright (c) 2016, 2018 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2016-2017 Łukasz Rogalski <rogalski.91@gmail.com>
 # Copyright (c) 2016-2017 Moises Lopez <moylop260@vauxoo.com>
-# Copyright (c) 2016 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2016 Brian C. Lane <bcl@redhat.com>
+# Copyright (c) 2017-2018 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2017 ttenhoeve-aa <ttenhoeve@appannie.com>
-# Copyright (c) 2017 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2018 Bryce Guinta <bryce.guinta@protonmail.com>
+# Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
+# Copyright (c) 2018 Ville Skyttä <ville.skytta@upcloud.com>
+# Copyright (c) 2018 Brian Shaginaw <brian.shaginaw@warbyparker.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -25,25 +29,12 @@
 # pylint: disable=W0611
 """some functions that may be useful for various checkers
 """
-import collections
-import functools
-try:
-    from functools import singledispatch as singledispatch
-except ImportError:
-    # pylint: disable=import-error
-    from singledispatch import singledispatch as singledispatch
-try:
-    from functools import lru_cache
-except ImportError:
-    from backports.functools_lru_cache import lru_cache
+import builtins
+from functools import lru_cache, partial, singledispatch
 import itertools
 import re
 import sys
 import string
-import warnings
-
-import six
-from six.moves import map, builtins # pylint: disable=redefined-builtin
 
 import astroid
 from astroid import bases as _bases
@@ -59,15 +50,17 @@ if not PY3K:
     EXCEPTIONS_MODULE = "exceptions"
 else:
     EXCEPTIONS_MODULE = "builtins"
-ABC_METHODS = set(('abc.abstractproperty', 'abc.abstractmethod',
-                   'abc.abstractclassmethod', 'abc.abstractstaticmethod'))
+ABC_METHODS = {'abc.abstractproperty', 'abc.abstractmethod',
+               'abc.abstractclassmethod', 'abc.abstractstaticmethod'}
 ITER_METHOD = '__iter__'
-NEXT_METHOD = 'next' if six.PY2 else '__next__'
+NEXT_METHOD = '__next__'
 GETITEM_METHOD = '__getitem__'
 SETITEM_METHOD = '__setitem__'
 DELITEM_METHOD = '__delitem__'
 CONTAINS_METHOD = '__contains__'
 KEYS_METHOD = 'keys'
+DATACLASS_DECORATOR = 'dataclass'
+DATACLASS_IMPORT = 'dataclasses'
 
 # Dictionary which maps the number of expected parameters a
 # special method can have to a set of special methods.
@@ -150,18 +143,18 @@ def clobber_in_except(node):
     (False, None) otherwise.
     """
     if isinstance(node, astroid.AssignAttr):
-        return (True, (node.attrname, 'object %r' % (node.expr.as_string(),)))
-    elif isinstance(node, astroid.AssignName):
+        return True, (node.attrname, 'object %r' % (node.expr.as_string(),))
+    if isinstance(node, astroid.AssignName):
         name = node.name
         if is_builtin(name):
             return (True, (name, 'builtins'))
-        else:
-            stmts = node.lookup(name)[1]
-            if (stmts and not isinstance(stmts[0].assign_type(),
-                                         (astroid.Assign, astroid.AugAssign,
-                                          astroid.ExceptHandler))):
-                return (True, (name, 'outer scope (line %s)' % stmts[0].fromlineno))
-    return (False, None)
+
+        stmts = node.lookup(name)[1]
+        if (stmts and not isinstance(stmts[0].assign_type(),
+                                     (astroid.Assign, astroid.AugAssign,
+                                      astroid.ExceptHandler))):
+            return True, (name, 'outer scope (line %s)' % stmts[0].fromlineno)
+    return False, None
 
 
 def is_super(node):
@@ -206,14 +199,10 @@ def is_defined_before(var_node):
     varname = var_node.name
     _node = var_node.parent
     while _node:
-        if isinstance(_node, COMP_NODE_TYPES):
+        if isinstance(_node, (COMP_NODE_TYPES, astroid.For)):
             for ass_node in _node.nodes_of_class(astroid.AssignName):
                 if ass_node.name == varname:
                     return True
-        elif isinstance(_node, astroid.For):
-            for ass_node in _node.target.nodes_of_class(astroid.AssignName):
-                if ass_node.name == varname:
-                    return True
         elif isinstance(_node, astroid.With):
             for expr, ids in _node.items:
                 if expr.parent_of(var_node):
@@ -237,7 +226,6 @@ def is_defined_before(var_node):
                 return True
             if getattr(_node, 'name', None) == varname:
                 return True
-            break
         elif isinstance(_node, astroid.ExceptHandler):
             if isinstance(_node.name, astroid.AssignName):
                 ass_node = _node.name
@@ -258,18 +246,19 @@ def is_defined_before(var_node):
         _node = _node.previous_sibling()
     return False
 
-def is_func_default(node):
-    """return true if the given Name node is used in function default argument's
-    value
+def is_default_argument(node):
+    """return true if the given Name node is used in function or lambda
+    default argument's value
     """
     parent = node.scope()
-    if isinstance(parent, astroid.FunctionDef):
+    if isinstance(parent, (astroid.FunctionDef, astroid.Lambda)):
         for default_node in parent.args.defaults:
             for default_name_node in default_node.nodes_of_class(astroid.Name):
                 if default_name_node is node:
                     return True
     return False
 
+
 def is_func_decorator(node):
     """return true if the name is used in function decorator"""
     parent = node.parent
@@ -285,7 +274,7 @@ def is_func_decorator(node):
     return False
 
 def is_ancestor_name(frame, node):
-    """return True if `frame` is a astroid.Class node with `node` in the
+    """return True if `frame` is an astroid.Class node with `node` in the
     subtree of its bases attribute
     """
     try:
@@ -467,6 +456,8 @@ def inherit_from_std_ex(node):
     if node.name in ('Exception', 'BaseException') \
             and node.root().name == EXCEPTIONS_MODULE:
         return True
+    if not hasattr(node, 'ancestors'):
+        return False
     return any(inherit_from_std_ex(parent)
                for parent in node.ancestors(recurs=True))
 
@@ -482,7 +473,7 @@ def error_of_type(handler, error_type):
     given errors.
     """
     def stringify_error(error):
-        if not isinstance(error, six.string_types):
+        if not isinstance(error, str):
             return error.__name__
         return error
 
@@ -548,8 +539,7 @@ def unimplemented_abstract_methods(node, is_abstract_cb=None):
     names and their inferred objects.
     """
     if is_abstract_cb is None:
-        is_abstract_cb = functools.partial(
-            decorated_with, qnames=ABC_METHODS)
+        is_abstract_cb = partial(decorated_with, qnames=ABC_METHODS)
     visited = {}
     try:
         mro = reversed(node.mro())
@@ -594,6 +584,7 @@ def unimplemented_abstract_methods(node, is_abstract_cb=None):
 
 
 def _import_node_context(node):
+    """Return the ExceptHandler or the TryExcept node in which the node is."""
     current = node
     ignores = (astroid.ExceptHandler, astroid.TryExcept)
     while current and not isinstance(current.parent, ignores):
@@ -625,7 +616,7 @@ def is_from_fallback_block(node):
 
 
 def _except_handlers_ignores_exception(handlers, exception):
-    func = functools.partial(error_of_type, error_type=(exception, ))
+    func = partial(error_of_type, error_type=(exception, ))
     return any(map(func, handlers))
 
 
@@ -640,17 +631,27 @@ def get_exception_handlers(node, exception):
         generator: the collection of handlers that are handling the exception or None.
 
     """
-    current = node
-    ignores = (astroid.ExceptHandler, astroid.TryExcept)
-    while current and not isinstance(current.parent, ignores):
-        current = current.parent
-
-    if current and isinstance(current.parent, astroid.TryExcept):
-        return (_handler for _handler in current.parent.handlers
+    context = _import_node_context(node)
+    if isinstance(context, astroid.TryExcept):
+        return (_handler for _handler in context.handlers
                 if error_of_type(_handler, exception))
     return None
 
 
+def is_node_inside_try_except(node):
+    """Check if the node is directly under a Try/Except statement.
+    (but not under an ExceptHandler!)
+
+    Args:
+        node (astroid.Raise): the node raising the exception.
+
+    Returns:
+        bool: True if the node is inside a try/except statement, False otherwise.
+    """
+    context = _import_node_context(node)
+    return isinstance(context, astroid.TryExcept)
+
+
 def node_ignores_exception(node, exception):
     """Check if the node is in a TryExcept which handles the given exception."""
     managing_handlers = get_exception_handlers(node, exception)
@@ -844,7 +845,7 @@ def is_none(node):
 def node_type(node):
     """Return the inferred type for `node`
 
-    If there is more than one possible type, or if inferred type is YES or None,
+    If there is more than one possible type, or if inferred type is Uninferable or None,
     return None
     """
     # check there is only one possible type for the assign node. Else we
@@ -889,6 +890,7 @@ def is_registered_in_singledispatch_function(node):
             continue
 
         if isinstance(func_def, astroid.FunctionDef):
+            # pylint: disable=redundant-keyword-arg; some flow inference goes wrong here
             return decorated_with(func_def, singledispatch_qnames)
 
     return False
@@ -915,3 +917,59 @@ def get_node_last_lineno(node):
         return get_node_last_lineno(node.body[-1])
     # Not a compound statement
     return node.lineno
+
+
+def is_enum_class(node):
+    """Check if a class definition defines an Enum class.
+
+    :param node: The class node to check.
+    :type node: astroid.ClassDef
+
+    :returns: True if the given node represents an Enum class. False otherwise.
+    :rtype: bool
+    """
+    for base in node.bases:
+        try:
+            inferred_bases = base.inferred()
+        except astroid.InferenceError:
+            continue
+
+        for ancestor in inferred_bases:
+            if not isinstance(ancestor, astroid.ClassDef):
+                continue
+
+            if ancestor.name == 'Enum' and ancestor.root().name == 'enum':
+                return True
+
+    return False
+
+
+def is_dataclass(node):
+    """Check if a class definition defines a Python 3.7+ dataclass
+
+    :param node: The class node to check.
+    :type node: astroid.ClassDef
+
+    :returns: True if the given node represents a dataclass class. False otherwise.
+    :rtype: bool
+    """
+    if not node.decorators:
+        return False
+    for decorator in node.decorators.nodes:
+        if not isinstance(decorator, (astroid.Name, astroid.Attribute)):
+            continue
+        if isinstance(decorator, astroid.Name):
+            name = decorator.name
+        else:
+            name = decorator.attrname
+        if name == DATACLASS_DECORATOR and DATACLASS_DECORATOR in node.root().locals:
+            return True
+    return False
+
+
+def is_postponed_evaluation_enabled(node):
+    """Check if the postponed evaluation of annotations is enabled"""
+    name = 'annotations'
+    module = node.root()
+    stmt = module.locals.get(name)
+    return stmt and isinstance(stmt[0], astroid.ImportFrom) and stmt[0].modname == '__future__'
