@@ -1,5 +1,15 @@
+# -*- coding: utf-8 -*-
+# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014-2015 Brett Cannon <brett@python.org>
-# Copyright (c) 2014-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
+# Copyright (c) 2015 Cosmin Poieana <cmin@ropython.org>
+# Copyright (c) 2015 Viorel Stirbu <viorels@gmail.com>
+# Copyright (c) 2016-2017 Roy Williams <roy.williams.iii@gmail.com>
+# Copyright (c) 2016 Roy Williams <rwilliams@lyft.com>
+# Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
+# Copyright (c) 2017 Daniel Miller <millerdev@gmail.com>
+# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2018 Ville Skyttä <ville.skytta@upcloud.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -25,6 +35,7 @@ python2_only = pytest.mark.skipif(sys.version_info[0] > 2, reason='Python 2 only
 # TODO(cpopa): Port these to the functional test framework instead.
 
 class TestPython3Checker(testutils.CheckerTestCase):
+
     CHECKER_CLASS = checker.Python3Checker
 
     def check_bad_builtin(self, builtin_name):
@@ -115,6 +126,11 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.walk(module)
 
+    def as_argument_to_materialized_filter(self, callable_fn):
+        module = astroid.parse("list(filter(None, {}()))".format(callable_fn))
+        with self.assertNoMessages():
+            self.walk(module)
+
     def as_argument_to_random_fxn_test(self, fxn):
         checker = '{}-builtin-not-iterating'.format(fxn)
         node = astroid.extract_node("""
@@ -160,25 +176,75 @@ class TestPython3Checker(testutils.CheckerTestCase):
         self.as_argument_to_str_join_test(fxn)
         self.as_iterable_in_unpacking(fxn)
         self.as_assignment(fxn)
+        self.as_argument_to_materialized_filter(fxn)
 
         for func in ('iter', 'list', 'tuple', 'sorted',
                      'set', 'sum', 'any', 'all',
                      'enumerate', 'dict'):
             self.as_argument_to_callable_constructor_test(fxn, func)
 
-    @python2_only
+    def test_dict_subclasses_methods_in_iterating_context(self):
+        iterating, not_iterating = astroid.extract_node('''
+        from __future__ import absolute_import
+        from collections import defaultdict
+        d = defaultdict(list)
+        a, b = d.keys() #@
+        x = d.keys() #@
+        ''')
+
+        with self.assertNoMessages():
+            self.checker.visit_call(iterating.value)
+
+        message = testutils.Message('dict-keys-not-iterating', node=not_iterating.value)
+        with self.assertAddsMessages(message):
+            self.checker.visit_call(not_iterating.value)
+
+    def test_dict_methods_in_iterating_context(self):
+        iterating_code = [
+            'for x in {}(): pass',
+            '(x for x in {}())',
+            '[x for x in {}()]',
+            'iter({}())',
+            'a, b = {}()',
+            'max({}())',
+            'min({}())',
+            '3 in {}()',
+            'set().update({}())',
+            '[].extend({}())',
+            '{{}}.update({}())',
+        ]
+        non_iterating_code = [
+            'x = __({}())',
+            '__({}())[0]',
+        ]
+
+        for method in ('keys', 'items', 'values'):
+            dict_method = '{{}}.{}'.format(method)
+
+            for code in iterating_code:
+                with_value = code.format(dict_method)
+                module = astroid.parse(with_value)
+                with self.assertNoMessages():
+                    self.walk(module)
+
+            for code in non_iterating_code:
+                with_value = code.format(dict_method)
+                node = astroid.extract_node(with_value)
+
+                checker = 'dict-{}-not-iterating'.format(method)
+                message = testutils.Message(checker, node=node)
+                with self.assertAddsMessages(message):
+                    self.checker.visit_call(node)
+
     def test_map_in_iterating_context(self):
         self.iterating_context_tests('map')
 
-    @python2_only
     def test_zip_in_iterating_context(self):
         self.iterating_context_tests('zip')
 
-    @python2_only
     def test_range_in_iterating_context(self):
         self.iterating_context_tests('range')
 
-    @python2_only
     def test_filter_in_iterating_context(self):
         self.iterating_context_tests('filter')
 
@@ -186,7 +252,7 @@ class TestPython3Checker(testutils.CheckerTestCase):
         """Helper for verifying that a certain method is not defined."""
         node = astroid.extract_node("""
             class Foo(object):
-                def __{0}__(self, other):  #@
+                def __{}__(self, other):  #@
                     pass""".format(method))
         message = testutils.Message(warning, node=node)
         with self.assertAddsMessages(message):
@@ -275,12 +341,18 @@ class TestPython3Checker(testutils.CheckerTestCase):
         message = testutils.Message('no-absolute-import', node=node)
         with self.assertAddsMessages(message):
             self.checker.visit_import(node)
+        with self.assertNoMessages():
+            # message should only be added once
+            self.checker.visit_import(node)
 
     def test_relative_from_import(self):
         node = astroid.extract_node('from os import path  #@')
         message = testutils.Message('no-absolute-import', node=node)
         with self.assertAddsMessages(message):
-            self.checker.visit_import(node)
+            self.checker.visit_importfrom(node)
+        with self.assertNoMessages():
+            # message should only be added once
+            self.checker.visit_importfrom(node)
 
     def test_absolute_import(self):
         module_import = astroid.parse(
@@ -332,10 +404,18 @@ class TestPython3Checker(testutils.CheckerTestCase):
                 self.checker.visit_call(node)
 
     def test_dict_iter_method_on_dict(self):
-        node = astroid.extract_node('{}.iterkeys()')
-        message = testutils.Message('dict-iter-method', node=node)
-        with self.assertAddsMessages(message):
-            self.checker.visit_call(node)
+        nodes = astroid.extract_node('''
+        from collections import defaultdict
+        {}.iterkeys() #@
+        defaultdict(list).iterkeys() #@
+        class Someclass(dict):
+            pass
+        Someclass().iterkeys() #@
+        ''')
+        for node in nodes:
+            message = testutils.Message('dict-iter-method', node=node)
+            with self.assertAddsMessages(message):
+                self.checker.visit_call(node)
 
     def test_dict_not_iter_method(self):
         arg_node = astroid.extract_node('x.iterkeys(x)  #@')
@@ -354,10 +434,18 @@ class TestPython3Checker(testutils.CheckerTestCase):
                 self.checker.visit_call(node)
 
     def test_dict_view_method_on_dict(self):
-        node = astroid.extract_node('{}.viewkeys()')
-        message = testutils.Message('dict-view-method', node=node)
-        with self.assertAddsMessages(message):
-            self.checker.visit_call(node)
+        nodes = astroid.extract_node('''
+        from collections import defaultdict
+        {}.viewkeys() #@
+        defaultdict(list).viewkeys() #@
+        class Someclass(dict):
+            pass
+        Someclass().viewkeys() #@
+        ''')
+        for node in nodes:
+            message = testutils.Message('dict-view-method', node=node)
+            with self.assertAddsMessages(message):
+                self.checker.visit_call(node)
 
     def test_dict_not_view_method(self):
         arg_node = astroid.extract_node('x.viewkeys(x)  #@')
@@ -409,7 +497,14 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_raise(node)
 
-    @python2_only
+    def test_xreadlines_attribute(self):
+        node = astroid.extract_node("""
+        f.xreadlines #@
+        """)
+        message = testutils.Message('xreadlines-attribute', node=node)
+        with self.assertAddsMessages(message):
+            self.checker.visit_attribute(node)
+
     def test_exception_message_attribute(self):
         node = astroid.extract_node("""
         try:
@@ -421,7 +516,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_attribute(node)
 
-    @python2_only
     def test_normal_message_attribute(self):
         node = astroid.extract_node("""
         e.message #@
@@ -429,33 +523,28 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.checker.visit_attribute(node)
 
-    @python2_only
     def test_invalid_codec(self):
         node = astroid.extract_node('foobar.encode("hex") #@')
         message = testutils.Message('invalid-str-codec', node=node)
         with self.assertAddsMessages(message):
             self.checker.visit_call(node)
 
-    @python2_only
     def test_valid_codec(self):
         node = astroid.extract_node('foobar.encode("ascii", "ignore")  #@')
         with self.assertNoMessages():
             self.checker.visit_call(node)
 
-    @python2_only
     def test_visit_call_with_kwarg(self):
         node = astroid.extract_node('foobar.raz(encoding="hex")  #@')
         with self.assertNoMessages():
             self.checker.visit_call(node)
 
-    @python2_only
     def test_invalid_open_codec(self):
         node = astroid.extract_node('open(foobar, encoding="hex") #@')
         message = testutils.Message('invalid-str-codec', node=node)
         with self.assertAddsMessages(message):
             self.checker.visit_call(node)
 
-    @python2_only
     def test_valid_open_codec(self):
         node = astroid.extract_node('open(foobar, encoding="palmos") #@')
         with self.assertNoMessages():
@@ -492,7 +581,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
             with self.assertAddsMessages(message):
                 self.checker.visit_call(node)
 
-    @python2_only
     def test_sys_maxint(self):
         node = astroid.extract_node('''
         import sys
@@ -502,7 +590,24 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_attribute(node)
 
-    @python2_only
+    def test_itertools_izip(self):
+        node = astroid.extract_node('''
+        from itertools import izip #@
+        ''')
+        absolute_import_message = testutils.Message('no-absolute-import', node=node)
+        message = testutils.Message('deprecated-itertools-function', node=node)
+        with self.assertAddsMessages(absolute_import_message, message):
+            self.checker.visit_importfrom(node)
+
+    def test_deprecated_types_fields(self):
+        node = astroid.extract_node('''
+        from types import StringType #@
+        ''')
+        absolute_import_message = testutils.Message('no-absolute-import', node=node)
+        message = testutils.Message('deprecated-types-field', node=node)
+        with self.assertAddsMessages(absolute_import_message, message):
+            self.checker.visit_importfrom(node)
+
     def test_sys_maxint_imort_from(self):
         node = astroid.extract_node('''
         from sys import maxint #@
@@ -512,7 +617,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(absolute_import_message, message):
             self.checker.visit_importfrom(node)
 
-    @python2_only
     def test_object_maxint(self):
         node = astroid.extract_node('''
         sys = object()
@@ -521,7 +625,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.checker.visit_attribute(node)
 
-    @python2_only
     def test_bad_import(self):
         node = astroid.extract_node('''
         import urllib2, sys #@
@@ -532,6 +635,18 @@ class TestPython3Checker(testutils.CheckerTestCase):
             self.checker.visit_import(node)
 
     @python2_only
+    def test_bad_import_not_on_relative(self):
+        samples = [
+            'from .commands import titi',
+            'from . import commands',
+        ]
+        for code in samples:
+            node = astroid.extract_node(code)
+            absolute_import_message = testutils.Message('no-absolute-import', node=node)
+            with self.assertAddsMessages(absolute_import_message):
+                self.checker.visit_importfrom(node)
+            self.checker._future_absolute_import = False
+
     def test_bad_import_conditional(self):
         node = astroid.extract_node('''
         import six
@@ -542,7 +657,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(absolute_import_message):
             self.checker.visit_import(node)
 
-    @python2_only
     def test_bad_import_try_except_handler(self):
         node = astroid.extract_node('''
         try:
@@ -554,7 +668,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(absolute_import_message):
             self.checker.visit_import(node)
 
-    @python2_only
     def test_bad_import_try(self):
         node = astroid.extract_node('''
         try:
@@ -568,7 +681,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(absolute_import_message):
             self.checker.visit_import(node)
 
-    @python2_only
     def test_bad_import_try_finally(self):
         node = astroid.extract_node('''
         try:
@@ -581,7 +693,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(absolute_import_message, message):
             self.checker.visit_import(node)
 
-    @python2_only
     def test_bad_import_from(self):
         node = astroid.extract_node('''
         from cStringIO import StringIO #@
@@ -591,7 +702,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(absolute_import_message, message):
             self.checker.visit_importfrom(node)
 
-    @python2_only
     def test_bad_string_attribute(self):
         node = astroid.extract_node('''
         import string
@@ -601,16 +711,96 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_attribute(node)
 
-    @python2_only
+    def test_bad_operator_attribute(self):
+        node = astroid.extract_node('''
+        import operator
+        operator.div #@
+        ''')
+        message = testutils.Message('deprecated-operator-function', node=node)
+        with self.assertAddsMessages(message):
+            self.checker.visit_attribute(node)
+
+    def test_comprehension_escape(self):
+        assign, escaped_node = astroid.extract_node('''
+        a = [i for i in range(10)] #@
+        i #@
+        ''')
+        good_module = astroid.parse('''
+        {c for c in range(10)} #@
+        {j:j for j in range(10)} #@
+        [image_child] = [x for x in range(10)]
+        thumbnail = func(__(image_child))
+        ''')
+        message = testutils.Message('comprehension-escape', node=escaped_node)
+        with self.assertAddsMessages(message):
+            self.checker.visit_listcomp(assign.value)
+
+        with self.assertNoMessages():
+            self.walk(good_module)
+
+    def test_comprehension_escape_newly_introduced(self):
+        node = astroid.extract_node('''
+        [i for i in range(3)]
+        for i in range(3):
+            i
+        ''')
+        with self.assertNoMessages():
+            self.walk(node)
+
+    def test_exception_escape(self):
+        module = astroid.parse('''
+        try: 1/0
+        except ValueError as exc:
+            pass
+        exc #@
+        try:
+           2/0
+        except (ValueError, TypeError) as exc:
+           exc = 2
+        exc #@
+        try:
+           2/0
+        except (ValueError, TypeError): #@
+           exc = 2
+        ''')
+        message = testutils.Message('exception-escape', node=module.body[1].value)
+        with self.assertAddsMessages(message):
+            self.checker.visit_excepthandler(module.body[0].handlers[0])
+        with self.assertNoMessages():
+            self.checker.visit_excepthandler(module.body[2].handlers[0])
+            self.checker.visit_excepthandler(module.body[4].handlers[0])
+
+    def test_bad_sys_attribute(self):
+        node = astroid.extract_node('''
+        import sys
+        sys.exc_clear #@
+        ''')
+        message = testutils.Message('deprecated-sys-function', node=node)
+        with self.assertAddsMessages(message):
+            self.checker.visit_attribute(node)
+
+    def test_bad_urllib_attribute(self):
+        nodes = astroid.extract_node('''
+        import urllib
+        urllib.addbase #@
+        urllib.splithost #@
+        urllib.urlretrieve #@
+        urllib.urlopen #@
+        urllib.urlencode #@
+        ''')
+        for node in nodes:
+            message = testutils.Message('deprecated-urllib-function', node=node)
+            with self.assertAddsMessages(message):
+                self.checker.visit_attribute(node)
+
     def test_ok_string_attribute(self):
         node = astroid.extract_node('''
         import string
-        string.letters #@
+        string.ascii_letters #@
         ''')
         with self.assertNoMessages():
             self.checker.visit_attribute(node)
 
-    @python2_only
     def test_bad_string_call(self):
         node = astroid.extract_node('''
         import string
@@ -620,7 +810,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_call(node)
 
-    @python2_only
     def test_ok_shadowed_call(self):
         node = astroid.extract_node('''
         import six.moves.configparser
@@ -629,7 +818,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.checker.visit_call(node)
 
-    @python2_only
     def test_ok_string_call(self):
         node = astroid.extract_node('''
         import string
@@ -638,7 +826,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.checker.visit_call(node)
 
-    @python2_only
     def test_bad_string_import_from(self):
         node = astroid.extract_node('''
          from string import atoi #@
@@ -648,7 +835,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(absolute_import_message, message):
             self.checker.visit_importfrom(node)
 
-    @python2_only
     def test_ok_string_import_from(self):
         node = astroid.extract_node('''
          from string import digits #@
@@ -657,7 +843,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(absolute_import_message):
             self.checker.visit_importfrom(node)
 
-    @python2_only
     def test_bad_str_translate_call_string_literal(self):
         node = astroid.extract_node('''
          foobar.translate(None, 'abc123') #@
@@ -667,7 +852,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_call(node)
 
-    @python2_only
     def test_bad_str_translate_call_variable(self):
         node = astroid.extract_node('''
          def raz(foobar):
@@ -678,7 +862,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_call(node)
 
-    @python2_only
     def test_bad_str_translate_call_infer_str(self):
         node = astroid.extract_node('''
          foobar = "hello world"
@@ -689,7 +872,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertAddsMessages(message):
             self.checker.visit_call(node)
 
-    @python2_only
     def test_ok_str_translate_call_integer(self):
         node = astroid.extract_node('''
          foobar.translate(None, 33) #@
@@ -697,7 +879,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.checker.visit_call(node)
 
-    @python2_only
     def test_ok_str_translate_call_keyword(self):
         node = astroid.extract_node('''
          foobar.translate(None, 'foobar', raz=33) #@
@@ -705,7 +886,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.checker.visit_call(node)
 
-    @python2_only
     def test_ok_str_translate_call_not_str(self):
         node = astroid.extract_node('''
          foobar = {}
@@ -740,7 +920,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.walk(module)
 
-    @python2_only
     def test_versioninfo_conditional(self):
         code = '''
         from __future__ import absolute_import
@@ -753,7 +932,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.walk(module)
 
-    @python2_only
     def test_versioninfo_tuple_conditional(self):
         code = '''
         from __future__ import absolute_import
@@ -766,7 +944,6 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.walk(module)
 
-    @python2_only
     def test_six_ifexp_conditional(self):
         code = '''
         from __future__ import absolute_import
@@ -778,13 +955,60 @@ class TestPython3Checker(testutils.CheckerTestCase):
         with self.assertNoMessages():
             self.walk(module)
 
+    def test_next_defined(self):
+        node = astroid.extract_node("""
+            class Foo(object):
+                def next(self):  #@
+                    pass""")
+        message = testutils.Message('next-method-defined', node=node)
+        with self.assertAddsMessages(message):
+            self.checker.visit_functiondef(node)
+
+    def test_next_defined_too_many_args(self):
+        node = astroid.extract_node("""
+            class Foo(object):
+                def next(self, foo=None):  #@
+                    pass""")
+        with self.assertNoMessages():
+            self.checker.visit_functiondef(node)
+
+    def test_next_defined_static_method_too_many_args(self):
+        node = astroid.extract_node("""
+            class Foo(object):
+                @staticmethod
+                def next(self):  #@
+                    pass""")
+        with self.assertNoMessages():
+            self.checker.visit_functiondef(node)
+
+    def test_next_defined_static_method(self):
+        node = astroid.extract_node("""
+            class Foo(object):
+                @staticmethod
+                def next():  #@
+                    pass""")
+        message = testutils.Message('next-method-defined', node=node)
+        with self.assertAddsMessages(message):
+            self.checker.visit_functiondef(node)
+
+    def test_next_defined_class_method(self):
+        node = astroid.extract_node("""
+            class Foo(object):
+                @classmethod
+                def next(cls):  #@
+                    pass""")
+        message = testutils.Message('next-method-defined', node=node)
+        with self.assertAddsMessages(message):
+            self.checker.visit_functiondef(node)
+
+
 @python2_only
 class TestPython3TokenChecker(testutils.CheckerTestCase):
 
     CHECKER_CLASS = checker.Python3TokenChecker
 
     def _test_token_message(self, code, symbolic_message):
-        tokens = testutils.tokenize_str(code)
+        tokens = testutils._tokenize_str(code)
         message = testutils.Message(symbolic_message, line=1)
         with self.assertAddsMessages(message):
             self.checker.process_tokens(tokens)
@@ -802,6 +1026,14 @@ class TestPython3TokenChecker(testutils.CheckerTestCase):
 
         # Make sure we are catching only octals.
         for non_octal in ("45", "00", "085", "08", "1"):
-            tokens = testutils.tokenize_str(non_octal)
+            tokens = testutils._tokenize_str(non_octal)
+            with self.assertNoMessages():
+                self.checker.process_tokens(tokens)
+
+    def test_non_ascii_bytes_literal(self):
+        code = 'b"测试"'
+        self._test_token_message(code, 'non-ascii-bytes-literal')
+        for code in ("测试", "测试", 'abcdef', b'\x80'):
+            tokens = testutils._tokenize_str(code)
             with self.assertNoMessages():
                 self.checker.process_tokens(tokens)
