@@ -1,8 +1,9 @@
 # Copyright (c) 2014 Vlad Temian <vladtemian@gmail.com>
-# Copyright (c) 2015-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2015-2017 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
 # Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
 # Copyright (c) 2017 guillaume2 <guillaume.peillex@gmail.col>
+# Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -11,7 +12,7 @@
 
 import json
 
-import six
+from io import StringIO
 
 from pylint.lint import PyLinter
 from pylint import checkers
@@ -19,7 +20,7 @@ from pylint.reporters.json import JSONReporter
 
 
 def test_simple_json_output():
-    output = six.StringIO()
+    output = StringIO()
 
     reporter = JSONReporter()
     linter = PyLinter(reporter=reporter)
