@@ -12,6 +12,9 @@
 # Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
 # Copyright (c) 2016 Roy Williams <roy.williams.iii@gmail.com>
 # Copyright (c) 2016 xmo-odoo <xmo-odoo@users.noreply.github.com>
+# Copyright (c) 2017 Bryce Guinta <bryce.paul.guinta@gmail.com>
+# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
+# Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -30,8 +33,7 @@ import sys
 import tempfile
 import tokenize
 
-import six
-from six.moves import StringIO
+from io import StringIO
 
 import astroid
 from pylint import checkers
@@ -162,7 +164,7 @@ class Message(collections.namedtuple('Message',
     __hash__ = None
 
 
-class UnittestLinter(object):
+class UnittestLinter:
     """A fake linter class to capture checker messages."""
     # pylint: disable=unused-argument, no-self-use
 
@@ -176,14 +178,16 @@ class UnittestLinter(object):
         finally:
             self._messages = []
 
-    def add_message(self, msg_id, line=None, node=None, args=None, confidence=None):
+    def add_message(self, msg_id, line=None, node=None, args=None, confidence=None,
+                    col_offset=None):
+        # Do not test col_offset for now since changing Message breaks everything
         self._messages.append(Message(msg_id, line, node, args, confidence))
 
     def is_message_enabled(self, *unused_args, **unused_kwargs):
         return True
 
     def add_stats(self, **kwargs):
-        for name, value in six.iteritems(kwargs):
+        for name, value in kwargs.items():
             self.stats[name] = value
         return self.stats
 
@@ -196,7 +200,7 @@ def set_config(**kwargs):
     def _wrapper(fun):
         @functools.wraps(fun)
         def _forward(self):
-            for key, value in six.iteritems(kwargs):
+            for key, value in kwargs.items():
                 setattr(self.checker.config, key, value)
             if isinstance(self, CheckerTestCase):
                 # reopen checker in case, it may be interested in configuration change
@@ -207,7 +211,7 @@ def set_config(**kwargs):
     return _wrapper
 
 
-class CheckerTestCase(object):
+class CheckerTestCase:
     """A base testcase class for unit testing individual checker classes."""
     CHECKER_CLASS = None
     CONFIG = {}
@@ -215,7 +219,7 @@ class CheckerTestCase(object):
     def setup_method(self):
         self.linter = UnittestLinter()
         self.checker = self.CHECKER_CLASS(self.linter) # pylint: disable=not-callable
-        for key, value in six.iteritems(self.CONFIG):
+        for key, value in self.CONFIG.items():
             setattr(self.checker.config, key, value)
         self.checker.open()
 
